import os
from .ignore import dir_ignore, files_ignore
import shutil
from pathlib import Path
from .utils import data_json_load, data_json_dump, hashing, current_version

def ready(current_dir :str, storage :str, house :str) -> None:
  directory = current_dir
  house_path = os.path.join(storage, house)
  temp = os.path.join(house_path, 'ready')
  json_file= os.path.join(house_path, 'data.json')
  created_directory = set()
  present_file_path = set()
  hash_data :dict = data_json_load(json_file)
  existing_files_with_hashes = set(hash_data.keys())
  for root, dirs, files in os.walk(directory):
    dirs[:] = [d for d in dirs if d not in dir_ignore]
    for file in files:
      if file in files_ignore:
        continue

      file_path = os.path.join(root, file)
      present_file_path.add(file_path)
      if hash_data:
        if hash_data.get(file_path):
          hash_value = hashing(file_path)
          if hash_data[file_path]['updated_hash'] == hash_value:
            continue
      parent = Path(directory)
      child = Path(root)
      relative = child.relative_to(parent)
      folder = os.path.join(temp, str(relative))
      if relative not in created_directory:
        os.makedirs(folder, exist_ok=True)
        created_directory.add(relative)
      shutil.copy2(file_path, folder)
  check_files_to_delete(house_path, json_file, existing_files_with_hashes, present_file_path, hash_data)

def check_files_to_delete(house_path :str, json_path :str ,existing_files_with_hashes :set, present_file_path :set, hash_data :dict):
  existing_files_with_hashes.remove("current_version")
  existing_files_with_hashes.remove("all_versions")
  files_deleted = existing_files_with_hashes - present_file_path
  if files_deleted:
    for file in files_deleted:
      hash_data[file]["deleted_in"] = int(current_version(house_path)) + 1
    data_json_dump(json_path, hash_data)



import os, hashlib, pickle, shutil, gzip
from .utils import update_version, data_json_dump, data_json_load

def if_directory_empty(directory :str) -> bool:
  if not os.path.isdir(directory):
    print(f"Error: '{directory}' is not a valid directory.")
    return False

  with os.scandir(directory) as it:
    return next(it, None) is None

def empty_ready_folder(directory :str) -> None:
  shutil.rmtree(directory)
  os.makedirs(directory)

def snapshot(current_directory :str, directory :str, current_house :str) -> None:
  working_directory = os.path.join(directory, current_house)
  ready_directory = os.path.join(working_directory, 'ready')
  if not if_directory_empty(ready_directory):
    snapshot_directory = os.path.join(working_directory, 'snapshot')
    json_file = os.path.join(working_directory, 'data.json')
    current_ver: str = update_version(working_directory)
    hash_data :dict= data_json_load(json_file)
    for root, dirs, files in os.walk(ready_directory):
      for file in files:
        snapshot_hash = hashlib.sha256()
        file_path = os.path.join(root, file)

        with open(file_path, 'rb') as f:
          snapshot_data = f.read()
        snapshot_hash.update(snapshot_data)
        hash_digest = snapshot_hash.hexdigest()
        file_path = file_path.replace(ready_directory, current_directory)
        if hash_data.__contains__(file_path):
          if hash_data[file_path]["updated_hash"] == hash_digest:
            continue
          hash_data[file_path]["updated_hash"] = hash_digest
          hash_data[file_path]["all_hashes"][current_ver] = hash_digest
        else:
          data = dict({"added_in": 0, "deleted_in": 0, "updated_hash": "", "all_hashes": {}})
          data["added_in"] = int(current_ver)
          data["updated_hash"] = hash_digest
          data["all_hashes"][current_ver] = hash_digest
          hash_data[file_path] = data
        save_file = os.path.join(snapshot_directory, hash_digest)
        with gzip.open(save_file, 'wb') as snap_file:
          pickle.dump(snapshot_data, snap_file)
    data_json_dump(json_file, hash_data)
    print(f'Snapshot created')
    empty_ready_folder(ready_directory)
  else:
    print("Nothing to Snapshot")


import os
from .utils import data_json_load, data_json_dump, add_files, delete_files

def revert_to_snapshot(directory :str, house :str, version :str) -> None:
  housing_path = os.path.join(directory, house)
  root_path = os.path.join(housing_path, 'snapshot')
  json_path = os.path.join(housing_path, 'data.json')
  json_data :dict = data_json_load(json_path)
  version_int = int(version)
  if int(version) not in json_data['all_versions']:
    print('Snapshot does not exist.')
    return
  if int(version) == json_data['current_version']:
    print(f"Already at {version}")
    return
  for file in json_data:
    if file == 'current_version':
      json_data['current_version'] = version_int
      continue
    elif file == 'all_versions':
      continue
    elif json_data[file]["added_in"] > version_int or json_data[file]["deleted_in"] < version_int and json_data[file]["deleted_in"] != 0:
      delete_files(file)
    else:
      file_hash = ""
      if json_data[file]["all_hashes"].__contains__(version):
        file_hash = json_data[file]["all_hashes"][version]
      else:
        list_data: dict = json_data[file]["all_hashes"]
        available_versions = map(int, list(list_data.keys()))
        available_versions = list(available_versions)
        for available in reversed(available_versions):
          if available < version_int:
            file_hash = list_data[f'{available}']
            break
      add_files(file, root_path, file_hash)
      json_data[file]["updated_hash"] = file_hash
  data_json_dump(json_path, json_data)
  print(f'Reverted to snapshot {version}')